@@ -19,7 +19,7 @@
 ;; URL:              https://github.com/meedstrom/el-job
 ;; Created:          2024-10-30
 ;; Keywords:         processes
-;; Package-Version:  2.4.3
+;; Package-Version:  2.4.4
 ;; Package-Requires: ((emacs "29.1"))
 
 ;;; Commentary:
@@ -35,7 +35,7 @@
 ;; You need to know the concept of a callback.
 
 ;; Public API:
-;; - Function `el-job-launch' (read its docstring)
+;; - Function `el-job-launch' (main entry point)
 ;; - Function `el-job-await'
 ;; - Function `el-job-is-busy'
 ;; - Variable `el-job-major-version'
@@ -109,13 +109,15 @@ an .eln anyway, without your having to recompile on save."
                                ;; Extra safety (sometimes files contain a patch
                                ;; overriding some other file's definition)
                                (string-prefix-p (symbol-name feature)
-                                                (symbol-name (cdr elem))))
+                                                (file-name-nondirectory file)))
                      return (let ((eln (native-comp-unit-file
                                         (subr-native-comp-unit
                                          (symbol-function (cdr elem))))))
                               ;; FIXME: comp sometimes deletes old eln during
                               ;; recompilation, but does not load the new eln,
-                              ;; at least not in a way that updates load-history
+                              ;; at least not in a way that updates
+                              ;; `load-history'.  Current workaround is return
+                              ;; nil, ie fall back on FILE; not ideal.
                               (when (file-exists-p eln)
                                 eln))))
        file)))
@@ -748,13 +750,37 @@ for buffer names starting with \" *el-job\" - note leading space."
      For example, first field is ID, second is CALLBACK etc."))))
 
 (defun el-job-kill-all ()
-  "Kill all el-job--all-jobs ever registered and forget metadata."
+  "Kill all el-jobs ever registered and forget metadata."
   (interactive)
   (maphash (lambda (id job)
              (el-job--disable job)
              (remhash id el-job--all-jobs))
            el-job--all-jobs))
 
+(defmacro el-job--sit-until-nil-p (test max-secs &optional message)
+  "Block until form TEST evaluates to nil, or time MAX-SECS has elapsed.
+Either way, return the last TEST result, i.e. non-nil if timed out.
+
+While blocking input, keep MESSAGE visible in the echo area.
+MESSAGE can be a string, or a form that evaluates to a string.
+
+Neither TEST nor MESSAGE should be expensive forms, since they are
+evaluated rapidly and cannot themselves trigger the time-out.
+A typical TEST would check if something in the environment has changed."
+  (let ((deadline (gensym "deadline"))
+        (last (gensym "last")))
+    `(let ((,deadline (time-add (current-time) ,max-secs))
+           ,last)
+       (catch 'timeout
+         (while (setq ,last ,test)
+           (unless (time-less-p (current-time) ,deadline)
+             (throw 'timeout nil))
+           ,(when message `(unless (current-message)
+                             (message "%s" ,message)))
+           (discard-input)
+           (sit-for 0.1)))
+       ,last)))
+
 (defun el-job-await (id max-secs &optional message)
   "Block until all processes for job ID finished, then return t.
 
@@ -762,19 +788,13 @@ If the job has still not finished after MAX-SECS seconds, stop
 blocking and return nil.
 
 Meanwhile, ensure string MESSAGE is visible in the minibuffer."
-  (let ((deadline (time-add (current-time) max-secs)))
-    (catch 'timeout
-      (while (el-job-is-busy id)
-        (discard-input)
-        (if (time-less-p (current-time) deadline)
-            (progn (unless (current-message) (message message))
-                   (sit-for 0.1))
-          (throw 'timeout nil)))
-      t)))
+  (not (el-job--sit-until-nil-p (el-job-is-busy id) max-secs message)))
 
 (defun el-job-is-busy (id)
   "Return list of busy processes for job ID, if any.
 Safely return nil otherwise, whether or not ID is known."
+  (when (el-job-p id)
+    (error "el-job-is-busy: Passed a job object, but expected only an ID"))
   (let ((job (gethash id el-job--all-jobs)))
     (and job (el-job-busy job))))
 
